#pragma once
#include <Windows.h>
#include <cstdint>
#include <map>
#include <set>
#include <boost/variant.hpp>
#include <boost/noncopyable.hpp>
#include "sp_exception.h"

class image_window;
class web_content_loader;

class configuration : boost::noncopyable
{
public:
	typedef std::set<std::string> string_set;

public:
	struct conf_property
	{
	public:
		typedef boost::variant<uint32_t, std::string, bool, string_set > value_type;

	public:
		template<typename T>
		conf_property(bool save_to_file, const T& value)
			:save_to_file_(save_to_file),
			prop_(value)
		{
		}

		conf_property();

		bool save_to_file() const;

		template<typename T>
		const T* get_value() const
		{
			return boost::get<T>(&prop_);
		}

		const value_type& get_variant() const;

	private:
		bool save_to_file_;
		value_type prop_;
	};

public:
	static void set_instance(HINSTANCE instance);
	static HINSTANCE get_instance();

	template<typename T>
	static const T get_property(const std::string& name, const T& default_value)
	{
		prop_map::const_iterator it = props_.find(name);
		if(it == props_.end())
			return default_value;

		const T* value = (*it).second.get_value<T>();
		if(!value)
			return default_value;

		return *value;
	}

	template<typename T>
	static void set_property(const std::string& name, const T& value, bool save_to_file)
	{
		if(!props_.insert(std::make_pair(name, conf_property(save_to_file, value))).second)
			throw sp_exception(sp_exception::unable_to_set_property, L"Duplicate property name");
	}

	template<typename T>
	static void replace_property(const std::string& name, const T& value, bool save_to_file)
	{
		props_[name] = conf_property(save_to_file, value);
	}
	
	static void clear(bool saved_only);
	static void load(const std::wstring& file_name, const std::wstring& app_name);
	static void save(const std::wstring& file_name, const std::wstring& app_name);

	static const std::wstring get_local_appdata_path();
	static const std::wstring prepare_settings_directory(const std::wstring& file_name, const std::wstring& app_name,
		bool create);
	
	static void apply_settings();
	static void set_loader_and_displayer(web_content_loader& loader, image_window& displayer);

private:
	configuration();
	static bool cast_to_bool(const std::string& value);

	static HINSTANCE instance_;

	typedef std::map<std::string, conf_property> prop_map;
	static prop_map props_;

	static web_content_loader* loader_;
	static image_window* displayer_;
};
