#include <Windows.h>
#include <CommCtrl.h>
#include <codecvt>
#include "dialogs.h"
#include "configuration.h"
#include "web_content_loader.h"
#include "skype.h"
#include "utility.h"
#include "resource.h"

void dialogs::show_settings_dialog(HWND owner)
{
	DialogBoxParamW(configuration::get_instance(), MAKEINTRESOURCEW(IDD_SETTINGS), owner, settings_dialog, IDD_SETTINGS);
}

void dialogs::show_about_dialog(HWND owner)
{
	DialogBoxParamW(configuration::get_instance(), MAKEINTRESOURCEW(IDD_ABOUT), owner, settings_dialog, IDD_ABOUT);
}

void dialogs::toggle_preview_contains_controls(HWND hWnd, bool enable)
{
	EnableWindow(GetDlgItem(hWnd, IDC_URL_CONTAINS_LIST), enable);
	if(enable)
		update_filter(hWnd, IDC_URL_CONTAINS_LIST, IDC_CONTAINS_LIST_EDIT, false);
	else
		EnableWindow(GetDlgItem(hWnd, IDC_CONTAINS_LIST_EDIT), FALSE);
}

void dialogs::toggle_preview_does_not_contain_controls(HWND hWnd, bool enable)
{
	EnableWindow(GetDlgItem(hWnd, IDC_URL_DOES_NOT_CONTAIN_LIST), enable);
	if(enable)
		update_filter(hWnd, IDC_URL_DOES_NOT_CONTAIN_LIST, IDC_DOES_NOT_CONTAIN_LIST_EDIT, false);
	else
		EnableWindow(GetDlgItem(hWnd, IDC_DOES_NOT_CONTAIN_LIST_EDIT), FALSE);
}

void dialogs::toggle_proxy_auth_controls(HWND hWnd, bool enable)
{
	EnableWindow(GetDlgItem(hWnd, IDC_PROXY_LOGIN), enable);
	EnableWindow(GetDlgItem(hWnd, IDC_PROXY_PASSWORD), enable);
}

void dialogs::toggle_proxy_controls(HWND hWnd, bool enable)
{
	EnableWindow(GetDlgItem(hWnd, IDC_PROXY_VALUE), enable);
	EnableWindow(GetDlgItem(hWnd, IDC_HTTP_PROXY), enable);
	EnableWindow(GetDlgItem(hWnd, IDC_SOCKS4), enable);
	EnableWindow(GetDlgItem(hWnd, IDC_SOCKS4A), enable);
	EnableWindow(GetDlgItem(hWnd, IDC_SOCKS5), enable);
	EnableWindow(GetDlgItem(hWnd, IDC_SOCKS5H), enable);
	EnableWindow(GetDlgItem(hWnd, IDC_USE_PROXY_AUTH), enable);
}

void dialogs::refresh_proxy_settings(HWND hWnd, bool auto_proxy, bool no_proxy, const std::wstring& proxy,
	bool use_proxy_auth, const std::wstring& proxy_login, const std::wstring& proxy_password)
{
	SendDlgItemMessageW(hWnd, IDC_SYSTEM_PROXY, BM_SETCHECK, BST_UNCHECKED, 0);
	SendDlgItemMessageW(hWnd, IDC_NO_PROXY, BM_SETCHECK, BST_UNCHECKED, 0);
	SendDlgItemMessageW(hWnd, IDC_MANUAL_PROXY, BM_SETCHECK, BST_UNCHECKED, 0);

	SendDlgItemMessageW(hWnd, IDC_USE_PROXY_AUTH, BM_SETCHECK, use_proxy_auth ? BST_CHECKED : BST_UNCHECKED, 0);
	SetDlgItemTextW(hWnd, IDC_PROXY_LOGIN, proxy_login.c_str());
	SetDlgItemTextW(hWnd, IDC_PROXY_PASSWORD, proxy_password.c_str());

	{
		web_content_loader::proxy_info info(web_content_loader::get_proxy_info(proxy));
		SetDlgItemTextW(hWnd, IDC_PROXY_VALUE, info.first.c_str());

		SendDlgItemMessageW(hWnd, IDC_SOCKS4, BM_SETCHECK, BST_UNCHECKED, 0);
		SendDlgItemMessageW(hWnd, IDC_SOCKS4A, BM_SETCHECK, BST_UNCHECKED, 0);
		SendDlgItemMessageW(hWnd, IDC_SOCKS5, BM_SETCHECK, BST_UNCHECKED, 0);
		SendDlgItemMessageW(hWnd, IDC_SOCKS5H, BM_SETCHECK, BST_UNCHECKED, 0);
		SendDlgItemMessageW(hWnd, IDC_HTTP_PROXY, BM_SETCHECK, BST_UNCHECKED, 0);

		switch(info.second)
		{
		case web_content_loader::proxy_http:
			SendDlgItemMessageW(hWnd, IDC_HTTP_PROXY, BM_SETCHECK, BST_CHECKED, 0);
			break;

		case web_content_loader::proxy_socks4:
			SendDlgItemMessageW(hWnd, IDC_SOCKS4, BM_SETCHECK, BST_CHECKED, 0);
			break;

		case web_content_loader::proxy_socks4a:
			SendDlgItemMessageW(hWnd, IDC_SOCKS4A, BM_SETCHECK, BST_CHECKED, 0);
			break;

		case web_content_loader::proxy_socks5:
			SendDlgItemMessageW(hWnd, IDC_SOCKS5, BM_SETCHECK, BST_CHECKED, 0);
			break;

		case web_content_loader::proxy_socks5h:
			SendDlgItemMessageW(hWnd, IDC_SOCKS5H, BM_SETCHECK, BST_CHECKED, 0);
			break;

		default:
			break;
		}
	}

	if(auto_proxy)
	{
		SendDlgItemMessageW(hWnd, IDC_SYSTEM_PROXY, BM_SETCHECK, BST_CHECKED, 0);
		toggle_proxy_controls(hWnd, false);
		toggle_proxy_auth_controls(hWnd, false);
	}
	else
	{
		if(no_proxy)
		{
			SendDlgItemMessageW(hWnd, IDC_NO_PROXY, BM_SETCHECK, BST_CHECKED, 0);
			toggle_proxy_controls(hWnd, false);
			toggle_proxy_auth_controls(hWnd, false);
		}
		else
		{
			SendDlgItemMessageW(hWnd, IDC_MANUAL_PROXY, BM_SETCHECK, BST_CHECKED, 0);
			toggle_proxy_controls(hWnd, true);
			toggle_proxy_auth_controls(hWnd, use_proxy_auth);
		}
	}
}

const std::string dialogs::get_field_value(HWND hWnd, uint32_t id)
{
	int text_length = GetWindowTextLengthW(GetDlgItem(hWnd, id));
	boost::scoped_array<wchar_t> buf(new wchar_t[text_length + 1]);
	buf[0] = L'\0';
	GetDlgItemTextW(hWnd, id, buf.get(), text_length + 1);
	std::wstring_convert<std::codecvt_utf8_utf16<wchar_t> > converter;
	return converter.to_bytes(buf.get());
}

void dialogs::init_settings(HWND hWnd)
{
	SetDlgItemInt(hWnd, IDC_CONNECTION_TIMEOUT,
		configuration::get_property("connection_timeout", skype::default_connection_timeout), FALSE);
	SetDlgItemInt(hWnd, IDC_TIMEOUT,
		configuration::get_property("read_timeout", skype::default_read_timeout), FALSE);
	SetDlgItemInt(hWnd, IDC_MAX_DATA,
		configuration::get_property("max_data_size", skype::default_max_data_size), FALSE);
	SetDlgItemInt(hWnd, IDC_MAXSIZE,
		configuration::get_property("max_image_x_y_size", skype::default_max_image_x_y_size), FALSE);
	SetDlgItemInt(hWnd, IDC_GIF_MAXSIZE,
		configuration::get_property("max_gif_x_y_size", skype::default_max_gif_x_y_size), FALSE);
	SetDlgItemInt(hWnd, IDC_GIF_MINTIME,
		configuration::get_property("min_frame_time", skype::default_min_frame_time), FALSE);
	SetDlgItemInt(hWnd, IDC_TITLE_LENGTH,
		configuration::get_property("max_title_length", skype::default_max_title_length), FALSE);
	SendDlgItemMessageW(hWnd, IDC_PLUGIN_ON, BM_SETCHECK,
		configuration::get_property("plugin_on", true) ? BST_CHECKED : BST_UNCHECKED, 0);
	SendDlgItemMessageW(hWnd, IDC_PREVIEW_PICTURES, BM_SETCHECK,
		configuration::get_property("preview_images", true) ? BST_CHECKED : BST_UNCHECKED, 0);
	SendDlgItemMessageW(hWnd, IDC_PREVIEW_TITLES, BM_SETCHECK,
		configuration::get_property("preview_titles", true) ? BST_CHECKED : BST_UNCHECKED, 0);
	SetDlgItemInt(hWnd, IDC_MAX_REDIRECTS,
		configuration::get_property("max_redirects", skype::default_max_redirects), FALSE);

	{
		bool filter_preview_contains = configuration::get_property("filter_preview_contains", false);
		bool filter_preview_does_not_contain = configuration::get_property("filter_preview_does_not_contain", false);
		SendDlgItemMessageW(hWnd, IDC_PREVIEW_CONTAINS, BM_SETCHECK,
			filter_preview_contains ? BST_CHECKED : BST_UNCHECKED, 0);
		SendDlgItemMessageW(hWnd, IDC_PREVIEW_DOES_NOT_CONTAIN, BM_SETCHECK,
			filter_preview_does_not_contain ? BST_CHECKED : BST_UNCHECKED, 0);

		toggle_preview_contains_controls(hWnd, filter_preview_contains);
		toggle_preview_does_not_contain_controls(hWnd, filter_preview_does_not_contain);
	}

	{
		const utility::encoding_list encodings(utility::get_supported_encoding_list());
		const std::string current_encoding(configuration::get_property("default_encoding", skype::default_encoding));
		for(utility::encoding_list::const_iterator it = encodings.begin(); it != encodings.end(); ++it)
			SendDlgItemMessageA(hWnd, IDC_DEFAULT_ENCODING, CB_ADDSTRING, 0, reinterpret_cast<LPARAM>((*it).c_str()));

		SendDlgItemMessageW(hWnd, IDC_DEFAULT_ENCODING, CB_INSERTSTRING, 0, reinterpret_cast<LPARAM>(L"<>"));
		if(current_encoding.empty())
		{
			SendDlgItemMessageW(hWnd, IDC_DEFAULT_ENCODING, CB_SETCURSEL, 0, 0);
		}
		else
		{
			SendDlgItemMessageA(hWnd, IDC_DEFAULT_ENCODING, CB_SELECTSTRING, static_cast<WPARAM>(-1),
				reinterpret_cast<LPARAM>(current_encoding.c_str()));
		}
	}

	{
		SendDlgItemMessageW(hWnd, IDC_MAX_DATA_DIMENSION, CB_SETCURSEL, 0, 0);
		uint32_t max_data_size = configuration::get_property("max_data_size", skype::default_max_data_size);
		for(uint32_t i = 1; i != 4; ++i)
		{
			if(max_data_size > 1024)
			{
				SendDlgItemMessageW(hWnd, IDC_MAX_DATA_DIMENSION, CB_SETCURSEL, i, 0);
				max_data_size /= 1024;
			}
		}

		SetDlgItemInt(hWnd, IDC_MAX_DATA, max_data_size, FALSE);
	}

	{
		bool auto_proxy = configuration::get_property("auto_proxy", true);
		bool no_proxy = configuration::get_property("no_proxy", true);
		const std::string proxy(configuration::get_property("proxy", std::string()));

		bool use_proxy_auth = configuration::get_property("proxy_auth", false);
		const std::string proxy_login(configuration::get_property("proxy_login", std::string()));
		const std::string proxy_password(configuration::get_property("proxy_password", std::string()));

		std::wstring_convert<std::codecvt_utf8_utf16<wchar_t> > converter;
		refresh_proxy_settings(hWnd, auto_proxy, no_proxy, converter.from_bytes(proxy),
			use_proxy_auth, converter.from_bytes(proxy_login),
			converter.from_bytes(proxy_password));

		SetDlgItemTextW(hWnd, IDC_USER_AGENT, 
			converter.from_bytes(configuration::get_property("user_agent", skype::default_user_agent)).c_str());
	}

	fill_combobox(hWnd, IDC_URL_CONTAINS_LIST, "url_contains_set");
	fill_combobox(hWnd, IDC_URL_DOES_NOT_CONTAIN_LIST, "url_does_not_contain_set");

	{
		bool preview_on_click = configuration::get_property("preview_on_click", false);
		SendDlgItemMessageW(hWnd, IDC_PREVIEW_MODE_CLICK, BM_SETCHECK, preview_on_click ? BST_CHECKED : BST_UNCHECKED, 0);
		SendDlgItemMessageW(hWnd, IDC_PREVIEW_MODE_MOUSEON, BM_SETCHECK, preview_on_click ? BST_UNCHECKED : BST_CHECKED, 0);
	}
}

void dialogs::fill_combobox(HWND hWnd, uint32_t id, const std::string& property_name)
{
	SendDlgItemMessageW(hWnd, id, CB_RESETCONTENT, 0, 0);
	configuration::string_set url_contains_set(configuration::get_property(property_name, configuration::string_set()));
	std::wstring_convert<std::codecvt_utf8_utf16<wchar_t> > converter;
	for(configuration::string_set::const_iterator it = url_contains_set.begin(); it != url_contains_set.end(); ++it)
	{
		SendDlgItemMessageW(hWnd, id, CB_ADDSTRING, 0,
			reinterpret_cast<LPARAM>(converter.from_bytes(*it).c_str()));
	}
}

const std::wstring dialogs::get_combobox_string(HWND hWnd, uint32_t id, int32_t index)
{
	int32_t string_length = SendDlgItemMessageA(hWnd, id, CB_GETLBTEXTLEN, static_cast<WPARAM>(index), 0);
	if(string_length == CB_ERR)
		return std::wstring();

	boost::scoped_array<wchar_t> buf(new wchar_t[string_length + 1]);
	if(CB_ERR != SendDlgItemMessageW(hWnd, id, CB_GETLBTEXT,
		static_cast<WPARAM>(index), reinterpret_cast<LPARAM>(buf.get())))
	{
		return buf.get();
	}

	return std::wstring();
}

bool dialogs::combobox_contains_string(HWND hWnd, uint32_t id, const std::wstring& value, int32_t* index)
{
	if(index)
		*index = 0;

	int32_t count = SendDlgItemMessageW(hWnd, id, CB_GETCOUNT, 0, 0);
	if(count == CB_ERR)
		return false;

	for(int32_t i = 0; i != count; ++i)
	{
		if(value == get_combobox_string(hWnd, id, i))
		{
			if(index)
				*index = i;

			return true;
		}
	}

	return false;
}

const configuration::string_set dialogs::get_combobox_strings(HWND hWnd, uint32_t id)
{
	configuration::string_set ret;
	int32_t count = SendDlgItemMessageW(hWnd, id, CB_GETCOUNT, 0, 0);
	if(count == CB_ERR)
		return ret;

	std::wstring_convert<std::codecvt_utf8_utf16<wchar_t> > converter;

	for(int32_t i = 0; i != count; ++i)
		ret.insert(converter.to_bytes(get_combobox_string(hWnd, id, i)));

	return ret;
}

const std::wstring dialogs::get_combo_text(HWND hWnd, uint32_t combo_id)
{
	HWND combo = GetDlgItem(hWnd, combo_id);
	int32_t length = GetWindowTextLengthW(combo);
	boost::scoped_array<wchar_t> buf(new wchar_t[length + 1]);
	GetWindowTextW(combo, buf.get(), length + 1);
	return buf.get();
}

void dialogs::update_filter(HWND hWnd, uint32_t combo_id, uint32_t button_id, bool on_select)
{
	std::wstring text;

	if(on_select)
	{
		text = get_combobox_string(hWnd, combo_id,
			SendDlgItemMessageW(hWnd, combo_id, CB_GETCURSEL, 0, 0));
	}
	else
	{
		text = get_combo_text(hWnd, combo_id);
	}

	if(combobox_contains_string(hWnd, combo_id, text, 0))
	{
		SetDlgItemTextW(hWnd, button_id, L"-");
		SetWindowLongPtrW(GetDlgItem(hWnd, button_id), GWLP_USERDATA, button_remove);
		EnableWindow(GetDlgItem(hWnd, button_id), TRUE);
	}
	else if(text.empty())
	{
		SetDlgItemTextW(hWnd, button_id, L"+");
		SetWindowLongPtrW(GetDlgItem(hWnd, button_id), GWLP_USERDATA, button_add);
		EnableWindow(GetDlgItem(hWnd, button_id), FALSE);
	}
	else
	{
		SetDlgItemTextW(hWnd, button_id, L"+");
		SetWindowLongPtrW(GetDlgItem(hWnd, button_id), GWLP_USERDATA, button_add);
		EnableWindow(GetDlgItem(hWnd, button_id), TRUE);
	}
}

INT_PTR WINAPI dialogs::settings_dialog(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{
	case WM_NOTIFY:
		switch(reinterpret_cast<LPNMHDR>(lParam)->code)
		{
		case NM_CLICK:
		case NM_RETURN:
			{
				const NMLINK* nmlink = reinterpret_cast<const NMLINK*>(lParam);

				if(nmlink->hdr.hwndFrom == GetDlgItem(hWnd, IDC_SITELINK))
					ShellExecuteW(NULL, L"open", L"http://kaimi.ru/", NULL, NULL, SW_SHOW);
			}
			break;
		}
		break;

	case WM_INITDIALOG:
		SetWindowLongPtrW(hWnd, GWLP_USERDATA, lParam);

		if(lParam != IDD_SETTINGS)
			break;

		SendDlgItemMessageW(hWnd, IDC_CONNECTION_TIMEOUT, EM_LIMITTEXT, 7, 0);
		SendDlgItemMessageW(hWnd, IDC_TIMEOUT, EM_LIMITTEXT, 7, 0);
		SendDlgItemMessageW(hWnd, IDC_MAX_DATA, EM_LIMITTEXT, 7, 0);
		SendDlgItemMessageW(hWnd, IDC_MAXSIZE, EM_LIMITTEXT, 7, 0);
		SendDlgItemMessageW(hWnd, IDC_GIF_MAXSIZE, EM_LIMITTEXT, 7, 0);
		SendDlgItemMessageW(hWnd, IDC_TITLE_LENGTH, EM_LIMITTEXT, 7, 0);
		SendDlgItemMessageW(hWnd, IDC_GIF_MINTIME, EM_LIMITTEXT, 7, 0);

		SendDlgItemMessageW(hWnd, IDC_TIMEOUT_SPIN, UDM_SETRANGE32, 1, 10000);
		SendDlgItemMessageW(hWnd, IDC_CONNECTION_TIMEOUT_SPIN, UDM_SETRANGE32, 1, 10000);
		SendDlgItemMessageW(hWnd, IDC_MAX_REDIRECTS_SPIN, UDM_SETRANGE32, 0, 100);

		SendDlgItemMessageW(hWnd, IDC_MAX_DATA_DIMENSION, CB_ADDSTRING, 0, reinterpret_cast<LPARAM>(L"B"));
		SendDlgItemMessageW(hWnd, IDC_MAX_DATA_DIMENSION, CB_ADDSTRING, 0, reinterpret_cast<LPARAM>(L"KB"));
		SendDlgItemMessageW(hWnd, IDC_MAX_DATA_DIMENSION, CB_ADDSTRING, 0, reinterpret_cast<LPARAM>(L"MB"));
		SendDlgItemMessageW(hWnd, IDC_MAX_DATA_DIMENSION, CB_ADDSTRING, 0, reinterpret_cast<LPARAM>(L"GB"));

		init_settings(hWnd);
		break;

	case WM_COMMAND:
		if(HIWORD(wParam) != 0)
		{
			if(HIWORD(wParam) == CBN_EDITUPDATE || HIWORD(wParam) == CBN_SELCHANGE || HIWORD(wParam) == CBN_SELENDCANCEL)
			{
				if(LOWORD(wParam) == IDC_URL_CONTAINS_LIST || LOWORD(wParam) == IDC_URL_DOES_NOT_CONTAIN_LIST)
				{
					update_filter(hWnd, LOWORD(wParam),
						LOWORD(wParam) == IDC_URL_CONTAINS_LIST ? IDC_CONTAINS_LIST_EDIT : IDC_DOES_NOT_CONTAIN_LIST_EDIT,
						HIWORD(wParam) == CBN_SELCHANGE);
				}
			}

			break;
		}

		switch(LOWORD(wParam))
		{
		case IDC_CONTAINS_LIST_EDIT:
		case IDC_DOES_NOT_CONTAIN_LIST_EDIT:
			{
				uint32_t combo_id =
					LOWORD(wParam) == IDC_CONTAINS_LIST_EDIT ? IDC_URL_CONTAINS_LIST : IDC_URL_DOES_NOT_CONTAIN_LIST;
				if(button_add == GetWindowLongPtrW(GetDlgItem(hWnd, LOWORD(wParam)), GWLP_USERDATA))
				{
					SendDlgItemMessageW(hWnd,
						combo_id,
						CB_ADDSTRING, 0,
						reinterpret_cast<LPARAM>(get_combo_text(hWnd, combo_id).c_str()));
				}
				else
				{
					std::wstring text(get_combo_text(hWnd, combo_id));
					int32_t index;
					if(combobox_contains_string(hWnd, combo_id, text, &index))
					{
						SendDlgItemMessageW(hWnd,
							combo_id,
							CB_DELETESTRING, index, 0);
					}
				}

				SetDlgItemTextW(hWnd, combo_id, L"");
				update_filter(hWnd, combo_id, LOWORD(wParam), false);
			}
			break;

		case IDC_RESTORE_DEFAULTS:
			if(IDYES == MessageBoxW(hWnd, L"        ?",
				L"", MB_YESNO | MB_ICONQUESTION))
			{
				try
				{
					configuration::clear(true);
					configuration::save(skype::settings_file_name, skype::plugin_name);
					configuration::apply_settings();

					init_settings(hWnd);
				}
				catch(const std::exception&)
				{
					MessageBoxW(hWnd, L"   ", L"", MB_ICONERROR | MB_OK);
				}
			}
			break;

		case IDC_PREVIEW_CONTAINS:
			toggle_preview_contains_controls(hWnd,
				SendDlgItemMessageW(hWnd, IDC_PREVIEW_CONTAINS, BM_GETCHECK, 0, 0) == BST_CHECKED);
			break;

		case IDC_PREVIEW_DOES_NOT_CONTAIN:
			toggle_preview_does_not_contain_controls(hWnd,
				SendDlgItemMessageW(hWnd, IDC_PREVIEW_DOES_NOT_CONTAIN, BM_GETCHECK, 0, 0) == BST_CHECKED);
			break;

		case IDC_USE_PROXY_AUTH:
			toggle_proxy_auth_controls(hWnd,
				SendDlgItemMessageW(hWnd, IDC_USE_PROXY_AUTH, BM_GETCHECK, 0, 0) == BST_CHECKED);
			break;

		case IDC_NO_PROXY:
		case IDC_SYSTEM_PROXY:
			toggle_proxy_controls(hWnd, false);
			toggle_proxy_auth_controls(hWnd, false);
			break;

		case IDC_MANUAL_PROXY:
			toggle_proxy_controls(hWnd, true);
			toggle_proxy_auth_controls(hWnd, SendDlgItemMessageW(hWnd, IDC_USE_PROXY_AUTH, BM_GETCHECK, 0, 0) == BST_CHECKED);
			break;

		case IDCANCEL:
			EndDialog(hWnd, 0);
			break;

		case IDOK:
			if(GetWindowLongPtrW(hWnd, GWLP_USERDATA) != IDD_SETTINGS)
			{
				EndDialog(hWnd, 0);
				break;
			}

			{
				uint32_t connection_timeout = GetDlgItemInt(hWnd, IDC_CONNECTION_TIMEOUT, NULL, FALSE);
				if(connection_timeout == 0)
				{
					MessageBoxW(hWnd, L"  .", L"", MB_ICONWARNING | MB_OK);
					break;
				}

				uint32_t read_timeout = GetDlgItemInt(hWnd, IDC_TIMEOUT, NULL, FALSE);
				if(read_timeout == 0)
				{
					MessageBoxW(hWnd, L"  .", L"", MB_ICONWARNING | MB_OK);
					break;
				}

				uint32_t max_data_size = GetDlgItemInt(hWnd, IDC_MAX_DATA, NULL, FALSE);
				if(max_data_size == 0)
				{
					MessageBoxW(hWnd, L"     .", L"", MB_ICONWARNING | MB_OK);
					break;
				}

				uint32_t max_image_x_y_size = GetDlgItemInt(hWnd, IDC_MAXSIZE, NULL, FALSE);
				if(max_image_x_y_size == 0)
				{
					MessageBoxW(hWnd, L"    .", L"", MB_ICONWARNING | MB_OK);
					break;
				}

				uint32_t max_gif_x_y_size = GetDlgItemInt(hWnd, IDC_GIF_MAXSIZE, NULL, FALSE);
				if(max_image_x_y_size == 0)
				{
					MessageBoxW(hWnd, L"     .", L"", MB_ICONWARNING | MB_OK);
					break;
				}

				uint32_t max_title_length = GetDlgItemInt(hWnd, IDC_TITLE_LENGTH, NULL, FALSE);
				if(max_title_length == 0)
				{
					MessageBoxW(hWnd, L"     .", L"", MB_ICONWARNING | MB_OK);
					break;
				}

				{
					bool value_ok = true;
					uint32_t cursel = SendDlgItemMessageW(hWnd, IDC_MAX_DATA_DIMENSION, CB_GETCURSEL, 0, 0);
					for(uint32_t i = 0; i != cursel; ++i)
					{
						uint32_t old_max_data_size = max_data_size;
						max_data_size *= 1024;
						if(max_data_size < old_max_data_size)
						{
							MessageBoxW(hWnd, L"      .", L"", MB_ICONWARNING | MB_OK);
							value_ok = false;
							break;
						}
					}

					if(!value_ok)
						break;
				}

				configuration::replace_property("connection_timeout", connection_timeout, true);
				configuration::replace_property("read_timeout", read_timeout, true);
				configuration::replace_property("max_data_size", max_data_size, true);
				configuration::replace_property("max_image_x_y_size", max_image_x_y_size, true);
				configuration::replace_property("max_gif_x_y_size", max_gif_x_y_size, true);
				configuration::replace_property("max_title_length", max_title_length, true);

				configuration::replace_property("max_redirects", GetDlgItemInt(hWnd, IDC_MAX_REDIRECTS, NULL, FALSE), true);

				int32_t selected_encoding = SendDlgItemMessageW(hWnd, IDC_DEFAULT_ENCODING, CB_GETCURSEL, 0, 0);
				if(selected_encoding != CB_ERR)
				{
					std::wstring encoding(get_combobox_string(hWnd, IDC_DEFAULT_ENCODING, selected_encoding));
					std::wstring_convert<std::codecvt_utf8_utf16<wchar_t> > converter;
					configuration::replace_property<std::string>("default_encoding", converter.to_bytes(encoding), true);
				}
				
				configuration::replace_property<uint32_t>("min_frame_time", 
					GetDlgItemInt(hWnd, IDC_GIF_MINTIME, NULL, FALSE), true);
			}
			
			configuration::replace_property("plugin_on", SendDlgItemMessageW(hWnd, IDC_PLUGIN_ON, BM_GETCHECK, 0, 0) == BST_CHECKED, true);
			configuration::replace_property("preview_images",
				SendDlgItemMessageW(hWnd, IDC_PREVIEW_PICTURES, BM_GETCHECK, 0, 0) == BST_CHECKED, true);
			configuration::replace_property("preview_titles",
				SendDlgItemMessageW(hWnd, IDC_PREVIEW_TITLES, BM_GETCHECK, 0, 0) == BST_CHECKED, true);
			
			configuration::replace_property("filter_preview_contains",
				SendDlgItemMessageW(hWnd, IDC_PREVIEW_CONTAINS, BM_GETCHECK, 0, 0) == BST_CHECKED, true);
			configuration::replace_property("filter_preview_does_not_contain",
				SendDlgItemMessageW(hWnd, IDC_PREVIEW_DOES_NOT_CONTAIN, BM_GETCHECK, 0, 0) == BST_CHECKED, true);

			{
				bool auto_proxy = SendDlgItemMessageW(hWnd, IDC_SYSTEM_PROXY, BM_GETCHECK, 0, 0) == BST_CHECKED;
				configuration::replace_property("auto_proxy", auto_proxy, true);
				bool no_proxy = SendDlgItemMessageW(hWnd, IDC_NO_PROXY, BM_GETCHECK, 0, 0) == BST_CHECKED;
				configuration::replace_property("no_proxy", no_proxy, true);
				bool use_proxy_auth = SendDlgItemMessageW(hWnd, IDC_USE_PROXY_AUTH, BM_GETCHECK, 0, 0) == BST_CHECKED;
				configuration::replace_property("proxy_auth", use_proxy_auth, true);
				
				std::string proxy(get_field_value(hWnd, IDC_PROXY_VALUE));
				if(SendDlgItemMessageW(hWnd, IDC_SOCKS4, BM_GETCHECK, 0, 0) == BST_CHECKED)
					proxy = "socks4://" + proxy;
				else if(SendDlgItemMessageW(hWnd, IDC_SOCKS4A, BM_GETCHECK, 0, 0) == BST_CHECKED)
					proxy = "socks4a://" + proxy;
				else if(SendDlgItemMessageW(hWnd, IDC_SOCKS5, BM_GETCHECK, 0, 0) == BST_CHECKED)
					proxy = "socks5://" + proxy;
				else if(SendDlgItemMessageW(hWnd, IDC_SOCKS5H, BM_GETCHECK, 0, 0) == BST_CHECKED)
					proxy = "socks5h://" + proxy;

				configuration::replace_property("proxy", proxy, true);
				configuration::replace_property("proxy_login", get_field_value(hWnd, IDC_PROXY_LOGIN), true);
				configuration::replace_property("proxy_password", get_field_value(hWnd, IDC_PROXY_PASSWORD), true);
			}

			configuration::replace_property("user_agent", get_field_value(hWnd, IDC_USER_AGENT), true);

			{
				configuration::string_set strings(get_combobox_strings(hWnd, IDC_URL_CONTAINS_LIST));
				configuration::replace_property("url_contains_set", strings, true);
				strings = get_combobox_strings(hWnd, IDC_URL_DOES_NOT_CONTAIN_LIST);
				configuration::replace_property("url_does_not_contain_set", strings, true);
			}
			
			configuration::replace_property("preview_on_click",
				SendDlgItemMessageW(hWnd, IDC_PREVIEW_MODE_CLICK, BM_GETCHECK, 0, 0) == BST_CHECKED, true);
			
			try
			{
				configuration::save(skype::settings_file_name, skype::plugin_name);
				configuration::apply_settings();
				EndDialog(hWnd, 0);
			}
			catch(const std::exception&)
			{
				MessageBoxW(hWnd, L"   ", L"", MB_ICONERROR | MB_OK);
			}
			break;
		}

		break;

	default:
		return FALSE;
	}

	return TRUE;
}
