#include "unpacked_image.h"

unpacked_image::unpacked_image(Gdiplus::Bitmap* single_bitmap)
	:min_frame_time_(0)
{
	frame_times_.push_back(0);
	frames_.push_back(single_bitmap);
}

uint32_t unpacked_image::get_frame_count() const
{
	return frame_times_.size();
}

void unpacked_image::add_image(Gdiplus::Image* image, uint32_t frame_time)
{
	frame_times_.push_back(frame_time);
	frames_.push_back(image);
}

Gdiplus::Image& unpacked_image::get_image(uint32_t index)
{
	return frames_.at(index);
}

uint32_t unpacked_image::get_frame_time(uint32_t index) const
{
	uint32_t ret = frame_times_.at(index);

	if(ret < min_frame_time_)
		ret = min_frame_time_;

	return ret;
}

void unpacked_image::set_min_frame_time(uint32_t min_frame_time)
{
	min_frame_time_ = min_frame_time;
}

unpacked_image::unpacked_image()
	:min_frame_time_(0)
{
}

std::auto_ptr<unpacked_image> unpacked_image::clone()
{
	std::auto_ptr<unpacked_image> ret(new unpacked_image());

	ret->frame_times_ = frame_times_;
	ret->min_frame_time_ = min_frame_time_;
	for(boost::ptr_vector<Gdiplus::Image>::iterator it = frames_.begin(); it != frames_.end(); ++it)
		ret->frames_.push_back((*it).Clone());

	return ret;
}
